{% extends "page.html" %}

{% load staticfiles %}

{% block styles %}
  {{ block.super }}
  <link href="{% static 'tethys_datasets/css/tethys_datasets.css' %}" rel="stylesheet" />
{% endblock %}

{% block primary_content %}
{% endblock %}

{% block scripts %}
  {{ block.super }}
  <script src="{% static 'tethys_datasets/js/tethys_datasets.js' %}" type="text/javascript"></script>
{% endblock %}